/************************************************************************/
/*  Computational Geometry Project                                      */
/*                                                                      */
/*  2 May 1995                                                          */
/*                                                                      */
/*  Paul Tyma                                                           */
/*  Vishal Mehra                                                        */
/*                                                                      */
/*  This code implements the Binary Space Partition algorithm along     */
/*  with our own enhancements.  This represents an O(n^2) worst case    */
/*  preprocessing step with O(n) worst case runtime step.               */
/*                                                                      */
/*  A rudimentary 3D graphical interface was built to support the       */
/*  above algorithm which allowed for transformation of the 3D          */
/*  polygons into 2D (i.e. an X-window) space.                          */
/*                                                                      */
/************************************************************************/


#define ALIVE_MASK	(SubstructureNotifyMask | KeyPressMask | PointerMotionMask)
#define HEIGHT 256
#define WIDTH  256
#define THRESH .2
#define UGH 99
#define DIST 150

#include <stdio.h>        
#include <stdlib.h>       
#include <math.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>

  
/************************************************************************/
/* The following includes (and several supporting functions were        */
/* taken from the book "XWindow System Programming" by Nabajyoti        */
/* Barkakati, ISBN: 0-672-30542-9                                       */
/*                                                                      */
/* The code used only performs administrative Xwindows tasks such as    */
/* Window creation and Palette setting.                                 */
/************************************************************************/

#include "xwins.h"
#include "app_glob.c"

/************************************************************************/
/* Setup global variables                                               */
/************************************************************************/

struct poly { int num;            /* number of points in this polygon */
              int *x,*y,*z;       /* coordinates of the polygon       */
	      int *cx,*cy,*cz;    /* transformed coordinates          */
	      int parent,kid[2];  /* parent and child pointers        */
	    } polygon[50];

struct junk { int x[6],y[6],z[6]; } special;
int    dir,bounce,numpts,ang,flag,root,prip;
float  posx,posz,cosdeg[360],sindeg[360];
int    pri[50];
XWMHints  *pxwm;
Window kb_win,root_win;
Pixmap pmap,pmap2;
char *quit_label = "Quit";
XEvent eventx;


/************************************************************************/
/* Function Prototypes                                                  */
/************************************************************************/

void initapp(/* int, char **, int, int, unsigned, unsigned */);
void CreateBSPTree();
void BuildPriorityList(int node);
int  LineInt(int g,int px1, int pz1, int px2, int pz2, int *x, int *y);
int  quit_action(/* caddr_t */);
void Splittree(int troot);
int  whichside(int a,int x,int z);
int  whichsidec(int a,int x,int z);
void Findsplits(int troot);
void updatemap();
int  dummytrue(void);
void transform(int rotatex,int rotatey,int rotatez,int g);
void modifyworld(KeySym inkey);
void DoCoolStuff(int rx,int ry,int rz);
void initview();

/************************************************************************/
/* MAIN                                                                 */
/************************************************************************/

main(argc, argv)
int  argc;
char **argv;
{
    XWIN     *QuitButton, *which_xwin;
    unsigned quit_width, quit_height;
    FILE *fp;
    int rx,ry,rz,g,l,h;    
    unsigned int depth;
    char  xlat[20];
    int   nchar=20;
    float e,f;
    int   count;
    KeySym key;
    XComposeStatus cs;

    bounce = 0;
    posx = 0.0;
    posz = 40.0;

/************************************************************************/
/* Put sines and cosines into a lookup table to speed processing        */
/************************************************************************/

    f = 0.0;
    e = 3.1415/180;
    for (g=0;g<360;++g)
      { sindeg[g] = sin(f);
	cosdeg[g] = cos(f);
	f += e;
      }

/************************************************************************/
/* Process input file                                                   */
/************************************************************************/

    fp = fopen(argv[1],"r");
   
    g = 0;
    numpts = 0;

    while (!feof(fp))
      { fscanf (fp, "%d ", &l);
	polygon[g].num = l;
        numpts++; 
        polygon[g].x = (int *) malloc (polygon[g].num * sizeof(int));
        polygon[g].y = (int *) malloc (polygon[g].num * sizeof(int));
        polygon[g].z = (int *) malloc (polygon[g].num * sizeof(int));

        polygon[g].cx = (int *) malloc (polygon[g].num * sizeof(int));
        polygon[g].cy = (int *) malloc (polygon[g].num * sizeof(int));
        polygon[g].cz = (int *) malloc (polygon[g].num * sizeof(int));
        for (h=0;h<polygon[g].num;++h)
	  {
	  fscanf(fp,"%d %d %d ",&rx,&ry,&rz);
	  polygon[g].x[h] = rx+1;
	  polygon[g].y[h] = ry+1;
	  polygon[g].z[h] = rz+1;
	}
	g++;
      }


/************************************************************************/
/* Routines from the book listed above, simply opens 3 Xwindows         */
/************************************************************************/
	 
    xwin_init();
    initapp(argc, argv, 0,0,256,256);

/************************************************************************/
/* Setup offscreen buffers (pixmaps) to use for drawing - this will     */
/* make the graphics "flicker-free"                                     */
/* Also, init the keyboard and further customize the window             */
/************************************************************************/

    depth = DefaultDepth(theDisplay, DefaultScreen(theDisplay));
    root_win = RootWindow(theDisplay, DefaultScreen(theDisplay));
    pmap = XCreatePixmap(theDisplay, root_win,256,256,depth);
    pmap2 = XCreatePixmap(theDisplay, root_win,HEIGHT,WIDTH,depth);
 
    ang = rx = ry = rz = 0;
    flag = 0;

    XSelectInput(theDisplay, theMain, ALIVE_MASK);

    pxwm = XAllocWMHints();
    pxwm->flags = (InputHint|StateHint);
    pxwm->input = True;
    pxwm->initial_state = NormalState;

    XSetWMHints(theDisplay,theMain,pxwm);

/************************************************************************/
/* Create a QUIT button on the first window for easy exiting            */
/************************************************************************/

    quit_width = XTextWidth(theFontStruct, quit_label, 
                            strlen(quit_label)) + 4;
    quit_height = theFontStruct->max_bounds.ascent + 
                  theFontStruct->max_bounds.descent + 4;

    QuitButton = MakeXButton(1, 1, quit_width, quit_height,
                             1, theFGpix, theBGpix, theMain, 
                             quit_label, quit_action, NULL);

    l = 0;
    modifyworld(0);
    updatemap();
    printf("Creating BSP Tree....\n");
    printf("There are %d polygons before BSP, ",numpts);
    CreateBSPTree();
    printf(" and %d after\n",numpts);
    printf("Initializing Views....\n");
    initview();

/************************************************************************/
/* Init the displays                                                    */
/************************************************************************/

    XSetForeground(theDisplay,theGC1,0);

    XDrawString(theDisplay,theMain1,theGC1,40,40,"Computational Geometry Project",30);
    XDrawString(theDisplay,theMain1,theGC1,85,100,"Spring 1995",11);
    XDrawString(theDisplay,theMain1,theGC1,95,150,"Paul Tyma",9);
    XDrawString(theDisplay,theMain1,theGC1,85,170,"Vishal Mehra",12);


    modifyworld(0);
    updatemap();
    DoCoolStuff(rx,ry,rz);
    dir = -4;
    ry = 169;

/************************************************************************/
/*  Start main message handling loop                                    */
/************************************************************************/

    printf("Event loop in control....\n");
    while (!AppDone)
    {

/************************************************************************/
/* Still messing around, watch for need to rotate an interior object    */
/************************************************************************/

	 if (flag && (l++>2000)) 
	     { l=0;
	       ry = ry+dir;
	       if ((ry<100) || (ry>170)) dir = -dir; 
	       
	       polygon[10].x[1] = polygon[10].x[2] = polygon[10].x[0] + sindeg[ry]*100;
	       polygon[10].z[1] = polygon[10].z[2] = polygon[10].z[0] + cosdeg[ry]*100;

	       modifyworld(0);	
	       updatemap();
	       DoCoolStuff(rx,ry,rz);
	      }

/************************************************************************/
/* Look for a message. We are being a bit nasty towards the "multi-     */
/* tasking" environment of X, this code will hog as much CPU as it is   */
/* allowed. This is need to facilitate the "walk-through" environ       */
/************************************************************************/

  if (XCheckIfEvent(theDisplay, &theEvent, dummytrue, NULL))
    {

	  switch(theEvent.type)
	    { case Expose : {
			     break;
			     }
	      case FocusIn:
			    XSetInputFocus(theDisplay, kb_win, RevertToParent,CurrentTime);
	                    break;

/************************************************************************/
/* Handle user keyboard input here                                      */
/************************************************************************/

	      case KeyPress :
		          { count = XLookupString(&theEvent,xlat,nchar,&key,&cs);
			    modifyworld(key);
			    updatemap();
			    DoCoolStuff(rx,ry,rz);
			    break;
			  }
               }

/************************************************************************/
/* Handle msgs for other windows                                        */
/************************************************************************/

        if (!(XFindContext(theDisplay, theEvent.xany.window, xwin_context, (caddr_t *) &which_xwin)))
            if(*(which_xwin->event_handler) != NULL)
              (*(which_xwin->event_handler))(which_xwin);
      } 
    } 

/************************************************************************/
/* Clean up memory and windows then exit                                */
/************************************************************************/

    printf("Exiting....\n");
    XFreePixmap(theDisplay, pmap);
    XCloseDisplay(theDisplay);
    for (g=0;g<numpts+2;++g)
       {
	 free(polygon[g].x);
	 free(polygon[g].y);
	 free(polygon[g].z);
         free(polygon[g].cx);
	 free(polygon[g].cy);
	 free(polygon[g].cz);
       }
  exit(0);
}

/************************************************************************/
/* Initial routine for BSP creation                                     */
/************************************************************************/

void CreateBSPTree()
{ int g,h;
 
/************************************************************************/
/* Initialize polygons to having no parent, call findroot to get first  */
/************************************************************************/

  for (g=0;g<numpts;++g)
    polygon[g].parent =-1;

  root = findroot(-1);

/************************************************************************/
/* Setup root and make it the universal parent                          */
/* Call 
/************************************************************************/

  polygon[root].kid[0] = polygon[root].kid[1] = -99;
  polygon[root].parent = -1;
  for (g=0;g<numpts;++g)
    if (g != root) polygon[g].parent = root;

  Splittree(root);

} 

/************************************************************************/
/* Find a suitable root                                                 */
/************************************************************************/

int findroot(int node)
    
{ int g,h,minx,miny,maxx,maxy;

  maxx = maxy = -30000;
  minx = miny = 30000;

return 5;
}

   

/************************************************************************/
/************************************************************************/
/* Routine to decide if a line segment intersects a line                */

/* g - polygon which represents infinite line                           */
/* px,pz - points of line segment                                       */
/* x,y - point of intersection                                          */
/************************************************************************/
/************************************************************************/

int LineInt(int g,int px1, int pz1, int px2, int pz2, int *x, int *y)
{ int dir;
  double den,ua,ub,p;
  long x21,y21,x43,y43,x13,y13;

  x21 = px2 - px1;
  y21 = pz2 - pz1;

  y43 = polygon[g].cz[2] - polygon[g].cz[0];
  x43 = polygon[g].cx[2] - polygon[g].cx[0];
  den = (y43 * x21 - x43 * y21);
  if (den == 0.0) return 0;      /* parallel lines do not intersect */

  y13 = pz1 - polygon[g].cz[0];
  x13 = px1 - polygon[g].cx[0];
  ub = ((double) x21 * y13 - y21 * x13 ) / den;

/************************************************************************/
/* Check if the intersection lies on the segment in question            */
/************************************************************************/

  if ((ub > 0.0) && (ub < 1.0))           
   { 
    ua = ((double) x43 * y13 - y43 * x13) / den;
    *x = px1 + ua * x21;
    *y = pz1 + ua * y21;
    return 1;
  }
 return 0;
}

/************************************************************************/
/************************************************************************/
/* Findsplits splits polygons when they are divided by the input line   */
/************************************************************************/
/************************************************************************/

void Findsplits(troot)
{ int g,h,dir;
  double den,ua,ub,p;
  long x21,y21,x43,y43,x13,y13,x,y;



  x21 = polygon[troot].x[2] - polygon[troot].x[0];
  y21 = polygon[troot].z[2] - polygon[troot].z[0];

  h = numpts;

/************************************************************************/
/* Loop through each applicable line (only those in the current segment)*/
/* and decide if there is a split                                       */
/************************************************************************/

  for (g=0;g<h;++g)
   {
    if (polygon[g].parent == troot)
      {
       y43 = polygon[g].z[2] - polygon[g].z[0];
       x43 = polygon[g].x[2] - polygon[g].x[0];
       den = (y43 * x21 - x43 * y21);
       if (den != 0.0)                       
	 {
           y13 = polygon[troot].z[0] - polygon[g].z[0];
           x13 = polygon[troot].x[0] - polygon[g].x[0];
           ub = ((double) x21 * y13 - y21 * x13 ) / den;

/************************************************************************/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/


	   /* Check if intersection lies on segment in question */
	   if ((ub > 0.0) && (ub < 1.0))           
	     { 
	       ua = ((double) x43 * y13 - y43 * x13) / den;
	       x = polygon[troot].x[0] + ua * x21;
	       y = polygon[troot].z[0] + ua * y21;

	       polygon[numpts].num = 4;
	       polygon[numpts].parent = troot;

               polygon[numpts].x = (int *) malloc (polygon[numpts].num * sizeof(int));
               polygon[numpts].y = (int *) malloc (polygon[numpts].num * sizeof(int));
               polygon[numpts].z = (int *) malloc (polygon[numpts].num * sizeof(int));

               polygon[numpts].cx = (int *) malloc (polygon[numpts].num * sizeof(int));
               polygon[numpts].cy = (int *) malloc (polygon[numpts].num * sizeof(int));
               polygon[numpts].cz = (int *) malloc (polygon[numpts].num * sizeof(int));


	       polygon[numpts].x[1] = polygon[g].x[1];
	       polygon[numpts].x[2] = polygon[g].x[2];
	       polygon[numpts].x[0] = polygon[g].x[1] = x;
	       polygon[numpts].x[3] = polygon[g].x[2] = x;

	       polygon[numpts].z[1] = polygon[g].z[1];
	       polygon[numpts].z[2] = polygon[g].z[2];
	       polygon[numpts].z[0] = polygon[g].z[1] = y;
	       polygon[numpts].z[3] = polygon[g].z[2] = y;

	       polygon[numpts].y[0] = polygon[g].y[0];
	       polygon[numpts].y[1] = polygon[g].y[1];
	       polygon[numpts].y[2] = polygon[g].y[2];
	       polygon[numpts].y[3] = polygon[g].y[3];
	       
	       numpts++;
	     }
	 }
 
     }

  }

}

/************************************************************************/
/* Splittree function recursively subdivides the polygon set and builds */
/* the BSP tree                                                         */
/************************************************************************/


void Splittree(int troot)
{ int g,h,t;

  Findsplits(troot);

/************************************************************************/
/* establish who's on the left and who's on the right                   */
/************************************************************************/

  for (g=0;g<numpts;++g)
    if (polygon[g].parent == troot)
	{ 
	  polygon[g].kid[0] = polygon[g].kid[1] = -99;

	  t = whichside(troot,(polygon[g].x[0]+polygon[g].x[2])/2,
			(polygon[g].z[0]+polygon[g].z[2])/2);
	  polygon[g].kid[t] = -1;

	}

/************************************************************************/
/* find a valid child and call recursively                              */
/************************************************************************/

 for (t=0;t<2;++t)
   {
    for (g=0,h=-1;g<numpts;++g)
     if ( (polygon[g].parent == troot) && (polygon[g].kid[t] == -1) )
       if (h<0) h=g;
	 else polygon[g].parent=h;

    if (h>-1)
     { polygon[troot].kid[t] = h;
       Splittree(h); 
      }
  }

}

/************************************************************************/
/* whichside returns which side a point lies with respect to a line     */
/* parms, number of the polygon, point x,y                              */
/* returns left = 0 ,  right = 1
/************************************************************************/

int whichside(int a,int x,int z)
  {  
    double ua,ub,p,sign;

          ua = polygon[a].x[2] - polygon[a].x[0];
          ub = polygon[a].z[2] - polygon[a].z[0];

	  if (ua == 0) return (x > polygon[a].x[0]);
	  if (ub == 0) return (z > polygon[a].z[0]);

          sign = ub/ua;

	  p  = (x - polygon[a].x[0])/ua;
	  p -= (z - polygon[a].z[0])/ub;
    
          return (p>0);
       }
/************************************************************************/
/* whichside returns which side a point lies with respect to a line     */
/* parms, number of the polygon, point x,y                              */
/* returns left = 0 ,  right = 1
/************************************************************************/

int whichsidec(int a,int x,int z)
  {  
    double ua,ub,p,sign;


          ua = polygon[a].cx[2] - polygon[a].cx[0];
          ub = polygon[a].cz[2] - polygon[a].cz[0];

	  if (ua == 0) return (x > polygon[a].cx[0]);
	  if (ub == 0) return (z > polygon[a].cz[0]);

          sign = ub/ua;

	  p  = (x - polygon[a].cx[0])/ua;
	  p -= (z - polygon[a].cz[0])/ub;
    
          return (p>0);
       }

/************************************************************************/
/* updates the 2D "overhead" view for the current viewpoint             */
/* this is immaterial as far as the workings of BSP but it provides a   */
/* a useful feature to the "realism" of the display                     */
/************************************************************************/

void updatemap()
{ int g,h,tx,tz,lx,lz,hx,hz,nseg=0;
  XSegment segments[100];
  
   XSetForeground(theDisplay,theGC2,0);
   XFillRectangle(theDisplay,pmap2,theGC2,0,0,256,256);
  
   XSetForeground(theDisplay,theGC2,4);
   XDrawPoint(theDisplay,pmap2,theGC2,127,127);
   XDrawPoint(theDisplay,pmap2,theGC2,127,128);
   XDrawPoint(theDisplay,pmap2,theGC2,128,127);
   XDrawPoint(theDisplay,pmap2,theGC2,128,128);


   for (g=0;g<numpts;++g)
     {
            XSetForeground(theDisplay,theGC2,1+(g>9)*(g+20));

	    lx = polygon[g].cx[0];
	    lz = -polygon[g].cz[0];
	    hx = polygon[g].cx[2];
	    hz = -polygon[g].cz[2];

	    XDrawLine(theDisplay,pmap2,theGC2,lx/10+128,lz/10+128,hx/10+128,hz/10+128);
      }

    XSetForeground(theDisplay,theGC2,200);
    XCopyArea(theDisplay, pmap2, theMain2, theGC2,0,0,256,256,0,0);
}

/************************************************************************/
/* modifyworld                                                          */
/* this proc handles key presses and acts upon them                     */
/* it recalculates the "world" according to a new position              */
/************************************************************************/



void modifyworld(KeySym inkey)
    
{ int g,h,x,y;
  float tx,tz,savex,savez;

  savex = posx;
  savez = posz;

  switch (inkey)
    { case XK_Up    : posx += 15.0*sindeg[ang];
                      posz -= 15.0*cosdeg[ang];
                      bounce = (bounce+25)%360;
                      break;
      case XK_Down  : posx -= 15.0*sindeg[ang];
                      posz += 15.0*cosdeg[ang];
                      bounce = (bounce+25)%360;
                      break;
      case XK_Left  : ang = (ang+5)%360;  
	              break;
      case XK_Right : ang = (ang-5);
                      if (ang<0) ang += 360;
	              break;
      case XK_r:
      case XK_R     : flag = (1-flag);
	              break;
    }


    for (g=0;g<numpts;++g)
      {
     for (h=0;h<polygon[g].num;++h)
       { tx = (float)polygon[g].x[h] + posx;
	 tz = (float)polygon[g].z[h] + posz;

         polygon[g].cx[h] = (float)cosdeg[ang] * tx + (float)sindeg[ang] * tz;
         polygon[g].cz[h] = (float)cosdeg[ang] * tz - (float)sindeg[ang] * tx;
	 polygon[g].cy[h] = polygon[g].y[h]+sindeg[bounce]*10;

       }
   }

/************************************************************************/
/* A quick and dirty method to prevent walking through walls            */
/************************************************************************/

    for (g=0;g<numpts;++g)
      if (LineInt(g,0,0,0,10,&x,&y))
	if ((float)abs(y) < 48.5) 
	  { posx = savex;
	    posz = savez;
	    return;
  	   }
}

/************************************************************************/
/* required for an Xwindows function                                    */
/************************************************************************/

Bool dummytrue()
{ return 1; }

/************************************************************************/
/* quit_action: called when quit button is pressed                      */
/************************************************************************/

int quit_action(data)
caddr_t data;
{
/* Set the done flag */
    AppDone = 1;
    return 1;

}


/************************************************************************/
/************************************************************************/
/* Main drawing routine for 3D screen                                   */
/* it first builds a priority list by traversing the BSP tree then      */
/* draws any polygons that are within the viewpoint                     */
/* for any polygons that are "partially" in the viewpoint, it clips the */
/* extraneous parts to prevent math overflows, etc.                     */
/************************************************************************/
/************************************************************************/


 void DoCoolStuff (int rx, int ry, int rz)
 { int g,h,i,tax,taz,tempz;
   double work,tz;
   XPoint point2d[10];

/************************************************************************/
/* recursively traverse the BSP to draw create priority list            */
/************************************************************************/

   prip = 0;

   BuildPriorityList(root);

/*
   printf("Current Priority List:  ");
   for (g=0;g<prip;++g)
     printf("%d ",pri[g]);
   printf("\n");
*/

/************************************************************************/
/* setup horizons in off screen buffer                                  */
/************************************************************************/


   XSetFillStyle(theDisplay,theGC,FillStippled);
   XSetForeground(theDisplay,theGC,255);
   XFillRectangle(theDisplay,pmap,theGC,0,0,256,128);

   XSetForeground(theDisplay,theGC,230);
   XFillRectangle(theDisplay,pmap,theGC,0,128,256,256);
   XSetFillStyle(theDisplay,theGC,FillSolid);

/************************************************************************/
/* loop through the polygons in the priority list                       */
/************************************************************************/

   for (i=0;i<prip;++i)
     {
     g = pri[i];  
     XSetForeground(theDisplay,theGC,g+1);  

/************************************************************************/
/* clip any polygons that cross the viewpoint lines                     */
/************************************************************************/

	 if (LineInt(g,polygon[numpts].cx[0],polygon[numpts].cz[0],
		     polygon[numpts].cx[2],polygon[numpts].cz[2],&tax,&taz))
	   { if (!(whichsidec(numpts,polygon[g].cx[0],polygon[g].cz[0])))
	       { polygon[g].cx[1] = polygon[g].cx[2] = tax;
		 polygon[g].cz[1] = polygon[g].cz[2] = taz;
	       }
	     else
	       { polygon[g].cx[0] = polygon[g].cx[3] = tax;
		 polygon[g].cz[0] = polygon[g].cz[3] = taz;
	       }
	   }

	 if (LineInt(g,polygon[numpts+1].cx[0],polygon[numpts+1].cz[0],
		     polygon[numpts+1].cx[2],polygon[numpts+1].cz[2],&tax,&taz))
	   { if (!(whichsidec(numpts+1,polygon[g].cx[0],polygon[g].cz[0])))
	       { polygon[g].cx[1] = polygon[g].cx[2] = tax;
		 polygon[g].cz[1] = polygon[g].cz[2] = taz;
	       }
	     else
	       { polygon[g].cx[0] = polygon[g].cx[3] = tax;
		 polygon[g].cz[0] = polygon[g].cz[3] = taz;
	       }
	   }

/************************************************************************/
/* only draw polygons that have some viewable portion                   */
/************************************************************************/

     tax = (polygon[g].cx[0] + polygon[g].cx[2])/2;
     taz = (polygon[g].cz[0] + polygon[g].cz[2])/2;
 
     if ( ( (tax >= -taz) && (tax <= taz) ) ||
          ( (polygon[g].cx[0] >= -polygon[g].cz[0]) && (polygon[g].cx[0] <= polygon[g].cz[0] ) ) ||
          ( (polygon[g].cx[2] >= -polygon[g].cz[2]) && (polygon[g].cx[2] <= polygon[g].cz[2] ) ) )
       {

/************************************************************************/
/* perform the transform from 2 to 3 dimensional space and draw         */
/************************************************************************/


     for (h=0;h<polygon[g].num;++h)
	    {

	     tz = polygon[g].cz[h]; 

	     if (tz < 0) tz =-tz;

	     work = DIST * ((double)polygon[g].cx[h]);
	     if (tz != 0) work /= tz;

	     work += 128;
	     
	     point2d[h].x = work;
	     work = DIST * ((double)polygon[g].cy[h]);
	     if (tz != 0) work /= tz;

	     work += 128;
	     
	     point2d[h].y = work;

	    }

          XFillPolygon(theDisplay,pmap,theGC,point2d,polygon[g].num,Nonconvex,CoordModeOrigin);

      }
   }

/************************************************************************/
/* copy the offscreen buffer to the window                              */
/************************************************************************/


   XCopyArea(theDisplay, pmap, theMain, theGC, 0,0,256,256,0,0);


 }

/************************************************************************/
/* BuildPriorityList:                                                   */
/* this proc recursively traverses the BSP tree and creates the list    */
/************************************************************************/

void BuildPriorityList(int node)
{
  int test,tax,taz,g,dumx,dumy,ff,hf,test1,slope;

  if (node<0) return;

  test = !(whichside(node,-posx,-posz));

  tax = (polygon[node].cx[0] + polygon[node].cx[2])/2;
  taz = (polygon[node].cz[0] + polygon[node].cz[2])/2;

  slope = (polygon[node].cx[2] - polygon[node].cx[0]);
  if (slope != 0)
     slope = (polygon[node].cz[2] - polygon[node].cz[0])/slope;
  else slope = 99;

  ff = (tax >= -taz);
  ff &= (tax <= taz);
  ff &= (taz > 0);


  hf = (LineInt(node,polygon[numpts].cx[0],polygon[numpts].cz[0],
		     polygon[numpts].cx[2],polygon[numpts].cz[2],&dumx,&dumy)) ;
  if (hf && (dumy > 0)) ff = 1;


  hf = (LineInt(node,polygon[numpts+1].cx[0],polygon[numpts+1].cz[0],
		     polygon[numpts+1].cx[2],polygon[numpts+1].cz[2],&dumx,&dumy)); 

  if (hf && (dumy > 0)) ff = 1;

/*  if ((slope > 1) || ( slope < -1)) ff=1; */

  test1 = whichsidec(node,0,0);
  if ((ang >90 ) && (ang < 181)) test1 = (1-test1);
  if ((ang >270 ) && (ang < 361)) test1 = (1-test1);


   BuildPriorityList(polygon[node].kid[test]);
   if (ff)  pri[prip++] = node;
   BuildPriorityList(polygon[node].kid[1-test]);

 }   


/************************************************************************/
/* initview:                                                            */
/* This proc initializes the "viewpoint" lines as polygons (they wont   */
/* be displayed in the 3D space) this facilitates drawing them in the   */
/* overhead map and for testing for intersecting polygons               */
/************************************************************************/



void initview()
{ int g;

	polygon[numpts].num = 4;
        polygon[numpts].x = (int *) malloc (4 * sizeof(int));
        polygon[numpts].y = (int *) malloc (4 * sizeof(int));
        polygon[numpts].z = (int *) malloc (4 * sizeof(int));

        polygon[numpts].cx = (int *) malloc (4 * sizeof(int));
        polygon[numpts].cy = (int *) malloc (4 * sizeof(int));
        polygon[numpts].cz = (int *) malloc (4 * sizeof(int));

	polygon[numpts+1].num = 4;
        polygon[numpts+1].x = (int *) malloc (4 * sizeof(int));
        polygon[numpts+1].y = (int *) malloc (4 * sizeof(int));
        polygon[numpts+1].z = (int *) malloc (4 * sizeof(int));

        polygon[numpts+1].cx = (int *) malloc (4 * sizeof(int));
        polygon[numpts+1].cy = (int *) malloc (4 * sizeof(int));
        polygon[numpts+1].cz = (int *) malloc (4 * sizeof(int));

        polygon[numpts].x[0] = polygon[numpts+1].x[0]   = 0; 
        polygon[numpts].z[0] = polygon[numpts+1].z[0] = 0; 
        polygon[numpts].y[0] = polygon[numpts+1].y[0] = 0; 

        polygon[numpts].x[3] = polygon[numpts+1].x[3]   = 0; 
        polygon[numpts].z[3] = polygon[numpts+1].z[3] = 0; 
        polygon[numpts].y[3] = polygon[numpts+1].y[3] = 0; 

        polygon[numpts].x[1] = 1000;
	polygon[numpts+1].x[1]   = -1000; 
        polygon[numpts].z[1] = polygon[numpts+1].z[1] = 1000; 
        polygon[numpts].y[1] = polygon[numpts+1].y[1] = 0; 

        polygon[numpts].x[2] = 1000;
	polygon[numpts+1].x[2]   = -1000; 
        polygon[numpts].z[2] = polygon[numpts+1].z[2] = 1000; 
        polygon[numpts].y[2] = polygon[numpts+1].y[2] = 0; 


       for (g=0;g<4;++g)
	 { polygon[numpts].cx[g] = polygon[numpts].x[g];
	   polygon[numpts].cy[g] = polygon[numpts].y[g];
	   polygon[numpts].cz[g] = polygon[numpts].z[g];
	   polygon[numpts+1].cx[g] = polygon[numpts+1].x[g];
	   polygon[numpts+1].cy[g] = polygon[numpts+1].y[g];
	   polygon[numpts+1].cz[g] = polygon[numpts+1].z[g];
	 }
}
